from urllib.error import HTTPError

from rest_framework.exceptions import ValidationError
from social_core.exceptions import MissingBackend, AuthTokenError
from social_django.utils import load_backend


def get_user_data_from_provider(strategy, provider, access_token, device_type):
    try:
        backend = load_backend(strategy=strategy, name=provider, redirect_uri=None)
    except MissingBackend:
        raise ValidationError({'provider': 'Please provide a valid provider'})
    try:
        user_data = backend.user_data(access_token, device_type=device_type)
    except HTTPError as error:
        raise ValidationError({'social_token': str(error)})
    except AuthTokenError as error:
        raise ValidationError({'social_token': str(error)})

    return user_data



import phonenumbers
from django.core.validators import EmailValidator
from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework_simplejwt.exceptions import AuthenticationFailed
from rest_framework_simplejwt.serializers import TokenObtainPairSerializer, TokenRefreshSerializer
from rest_framework_simplejwt.tokens import RefreshToken

from drf_auth_service.common.enums import RegisterType, LogsType
from drf_auth_service.models import ActivationCode, UserLogs
from drf_auth_service.settings import User
from drf_auth_service.settings import settings
from drf_auth_service.validators import validate_token_by_type


class RegisterSerializer(serializers.Serializer):
    username = serializers.CharField(required=True)
    password = serializers.CharField(required=True)
    register_type = serializers.ChoiceField(choices=settings.ENUMS.REGISTER_TYPES.choices, required=True)

    def validate(self, attrs):
        error_message = ValidationError({"username": f"Value '{attrs['username']}' is invalid phone number"})

        if User.objects.filter(username=attrs['username'], service=self.context['request'].service).exists():
            raise ValidationError({'username': f"User '{attrs['username']}' already exists"})

        if attrs['register_type'] == RegisterType.EMAIL:
            email_validator = EmailValidator({"username": f"Value '{attrs['username']}' is invalid email address"})
            email_validator(attrs['username'])

        elif attrs['register_type'] == RegisterType.PHONE_NUMBER or \
                attrs['register_type'] == RegisterType.PHONE_NUMBER_CODE:

            try:
                phone = phonenumbers.parse(f"+{attrs['username']}", None)
            except:
                raise error_message

            if not phonenumbers.is_valid_number(phone):
                raise error_message
        return attrs


class ReturnAccessTokenSerializer(serializers.Serializer):
    access_token = serializers.CharField(label='Jwt token for authentication')


class ReturnJWTTokensSerializer(ReturnAccessTokenSerializer):
    refresh_token = serializers.CharField(label='Refresh token for access token')


class ReturnRegisterSerializer(ReturnAccessTokenSerializer):
    refresh_token = serializers.CharField(label='Jwt token for refresh access token')


class ReturnSuccessSerializer(serializers.Serializer):
    success = serializers.BooleanField(default=True)
    message = serializers.CharField(label='Here will be displayed message of success or fail')


class SendResetPasswordSerializer(serializers.Serializer):
    username = serializers.CharField(required=True)

    def validate(self, attrs):
        attrs['user'] = User.objects.filter(username=attrs['username']).first()
        if attrs['user'] is None:
            raise ValidationError({'username': 'Invalid username'})

        return attrs


class ResetPasswordConfirmSerializer(serializers.Serializer):
    password = serializers.CharField(label="Password", required=True, allow_null=False, allow_blank=False)
    confirm_password = serializers.CharField(label="Confirm password", required=True,
                                             allow_null=False, allow_blank=False)
    token = serializers.CharField(label="Reset password token from url", required=True, allow_null=False,
                                  allow_blank=False)

    def validate(self, data):
        """
        Check if passwords matched and if token is valid.
        """
        data['token'] = validate_token_by_type(data['token'], ActivationCode.CodeTypes.RESET_PASSWORD)
        if data['password'] != data['confirm_password']:
            raise serializers.ValidationError("Passwords does not match")
        return data


class LoginSerializer(TokenObtainPairSerializer):
    @classmethod
    def get_token(cls, user):
        token = super().get_token(user)

        token['username'] = user.username
        token['user_id'] = user.id

        return token

    def validate(self, attrs):
        data = super().validate(attrs)
        if getattr(self.context['request'], 'service', None):
            UserLogs.objects.create(service=self.context['request'].service, user=self.user,
                                    username=self.user.username, type_log=LogsType.LOGIN_BY_CREDENTIALS)
        return data


class EBSTokenRefreshSerializer(TokenRefreshSerializer):
    def validate(self, attrs):
        refresh = RefreshToken(attrs['refresh'])

        user = User.objects.filter(
            pk=refresh.payload.get("user_id"),
            service=self.context['request'].service
        ).first()

        if not user:
            raise AuthenticationFailed('User is inactive', code='user_inactive')

        if user.is_blocked:
            raise AuthenticationFailed('User is blocked', code='user_blocked')

        if not user.is_confirmed:
            raise AuthenticationFailed('User is not confirmed', code='user_not_confirmed')

        UserLogs.objects.create(
            service=self.context['request'].service,
            user=user,
            username=user,
            type_log=LogsType.LOGIN_BY_CREDENTIALS
        )

        data = super(EBSTokenRefreshSerializer, self).validate(attrs)
        data['access']['username'] = user.username

        return data



from django.urls import path
from rest_framework.routers import DefaultRouter

from drf_auth_service.authentication.views import LoginViewSet, EBSTokenRefreshView
from drf_auth_service.settings import settings
from drf_auth_service.socials.views import social_login

router = DefaultRouter(trailing_slash=False)
router.register(r'services', settings.VIEWS.SERVICE_VIEWS, basename='service')
router.register(r'users', settings.VIEWS.USER_VIEWS, basename='user')
router.register(r'authentication', settings.VIEWS.AUTHENTICATION_VIEWS, basename='authentication')

urlpatterns = router.urls + [
    path('authentication/login', LoginViewSet.as_view(), name='token_obtain_pair'),
    path('authentication/oauth', social_login, name='social_login'),
    path('authentication/refresh', EBSTokenRefreshView.as_view(), name='token_refresh')
]



from datetime import timedelta

from django.utils import timezone
from rest_framework.exceptions import ValidationError
from rest_framework.generics import get_object_or_404

from drf_auth_service.models import ActivationCode
from drf_auth_service.settings import User


def validate_token_by_type(token, code_type):
    password_reset_token_validation_time = 24

    reset_password_token = get_object_or_404(ActivationCode, key=token, key_type=code_type)

    expiry_date = reset_password_token.created_at + timedelta(hours=password_reset_token_validation_time)

    if timezone.now() > expiry_date:
        reset_password_token.delete()
        raise ValidationError(dict(detail='Token expired'))

    return reset_password_token


def validate_user(user_filter):
    user = User.objects.filter(username=user_filter).first()
    if user is None:
        raise ValidationError('User does not exist')

    return user



from rest_framework import serializers

from drf_auth_service.models import Social
from drf_auth_service.settings import settings
from drf_auth_service.socials.constants import DeviceTypes


class ReturnUserSocialDataSerializer(serializers.Serializer):
    id = serializers.IntegerField(label='Social Id', required=False)
    first_name = serializers.CharField(label='Social first name', required=False)
    last_name = serializers.CharField(label='Social last name', required=False)
    picture = serializers.ImageField(label='Social image', required=False)
    email = serializers.ImageField(label='Social email', required=False)


class ReturnSocialSerializer(serializers.Serializer):
    data = ReturnUserSocialDataSerializer()
    exists = serializers.BooleanField(label='True if user with this email or social'
                                            ' identifier already exists in our database')


class SocialSerializer(serializers.Serializer):
    """
    Serializer which accepts an OAuth2 access token and provider.
    """

    social_token = serializers.CharField(required=True, allow_blank=False, trim_whitespace=True)
    provider = serializers.ChoiceField(required=True, allow_blank=False, allow_null=False,
                                       choices=settings.ENUMS.SOCIAL_TYPES)
    device_type = serializers.ChoiceField(choices=DeviceTypes.choices, default=DeviceTypes.ANDROID)


class SocialModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = Social
        fields = ('label', 'identifier', 'social_type')



from rest_framework import serializers

from drf_auth_service.models import Service


class ServiceSerializer(serializers.ModelSerializer):
    public_token = serializers.CharField(read_only=True)
    secret_token = serializers.CharField(read_only=True)

    class Meta:
        model = Service
        fields = '__all__'



from django.db import models
from django.utils import timezone


class BaseModel(models.Model):
    modified_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(default=timezone.now)

    class Meta:
        abstract = True



from datetime import timedelta

from django.conf import settings as django_settings
from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext as _

from drf_auth_service.common.enums import RegisterType, SocialTypes, LogsType
from drf_auth_service.common.helpers import generate_token, generate_code
from drf_auth_service.common.models import BaseModel


class Service(models.Model):
    name = models.CharField(max_length=30)
    secret_token = models.CharField(unique=True, max_length=200, default=generate_token)
    public_token = models.CharField(unique=True, max_length=200, default=generate_token)

    class Meta:
        db_table = '"services"'


class AbstractSSOUser(AbstractBaseUser):
    username = models.CharField(max_length=200, unique=True)
    service = models.ForeignKey(Service, on_delete=models.CASCADE, related_name='service_users')
    register_type = models.CharField(max_length=100, choices=RegisterType.choices, default=RegisterType.EMAIL)
    USERNAME_FIELD = 'username'
    objects = BaseUserManager()

    class Meta:
        db_table = '"users"'
        unique_together = [['username', 'service']]
        abstract = True

    @property
    def is_confirmed(self):
        if self.activation_code.filter(key_type=ActivationCode.CodeTypes.CONFIRM_ACCOUNT).first() is None:
            return True
        return False

    @property
    def is_blocked(self):
        try:
            if self.block:
                return True
        except ObjectDoesNotExist:
            return False


class SSOUser(AbstractSSOUser):
    class Meta(AbstractSSOUser.Meta):
        swappable = 'AUTH_USER_MODEL'


class Scope(models.Model):
    description = models.TextField()
    name = models.CharField(max_length=50)
    code = models.CharField(max_length=200)
    service = models.ForeignKey(Service, on_delete=models.CASCADE)

    class Meta:
        db_table = '"scopes"'


class Role(models.Model):
    scopes = models.ManyToManyField(Scope)
    name = models.CharField(max_length=300)
    description = models.CharField(max_length=300)
    service = models.ForeignKey(Service, on_delete=models.CASCADE)

    class Meta:
        db_table = '"roles"'


class UserBlock(BaseModel):
    ip = models.CharField(max_length=50, null=True)
    reason = models.CharField(max_length=254, null=True)
    user = models.OneToOneField(django_settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='block')

    class Meta:
        db_table = '"user_blocks"'


class Social(BaseModel):
    label = models.CharField(max_length=100)
    identifier = models.CharField(max_length=100)
    social_type = models.CharField(max_length=20, choices=SocialTypes.choices)
    user = models.ForeignKey(django_settings.AUTH_USER_MODEL, on_delete=models.CASCADE,
                             related_name='user_social_identifier')

    class Meta:
        unique_together = [['user', 'social_type']]
        db_table = '"socials"'


class Config(BaseModel):
    key = models.CharField(max_length=200)
    type = models.CharField(max_length=200)
    value = models.TextField()
    service = models.ForeignKey(Service, on_delete=models.CASCADE, related_name='configs')

    class Meta:
        db_table = '"configs"'
        unique_together = [['service', 'key']]


class ActivationCode(models.Model):
    class CodeTypes(models.TextChoices):
        RESET_PASSWORD = 'reset_password', _('Reset password')
        CONFIRM_ACCOUNT = 'confirm_account', _('Confirm account')

    created_at = models.DateTimeField(auto_now_add=True)
    key = models.CharField(max_length=64, db_index=True, unique=True)
    user = models.ForeignKey(django_settings.AUTH_USER_MODEL, related_name='activation_code', on_delete=models.CASCADE)
    key_type = models.CharField(max_length=30, choices=CodeTypes.choices, default=CodeTypes.RESET_PASSWORD)

    class Meta:
        db_table = '"activation_codes"'

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = generate_token()

        return super(ActivationCode, self).save(*args, **kwargs)

    @classmethod
    def clear_expired(cls, expiry_time, key_type=None):
        if key_type is None:
            cls.objects.filter(created_at__lte=expiry_time, key_type=cls.CodeTypes.RESET_PASSWORD).delete()

        elif key_type == cls.CodeTypes.CONFIRM_ACCOUNT:
            cls.objects.filter(created_at__lte=expiry_time, key_type=cls.CodeTypes.CONFIRM_ACCOUNT).delete()

    # TODO think about change
    @classmethod
    def make_user_active(cls, user):
        cls.objects.filter(user=user, key_type=cls.CodeTypes.CONFIRM_ACCOUNT).delete()

    @staticmethod
    def create_token(user, key_type, for_phone=False):
        password_reset_token_validation_time = 24
        now_minus_expiry_time = timezone.now() - timedelta(hours=password_reset_token_validation_time)
        ActivationCode.clear_expired(now_minus_expiry_time, key_type=key_type)
        key = generate_code() if for_phone else generate_token()
        token, is_created = ActivationCode.objects.update_or_create(user=user, key_type=key_type, defaults={'key': key})
        return token


class UserLogs(models.Model):
    username = models.CharField(max_length=200)
    user = models.ForeignKey(django_settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    type_log = models.CharField(max_length=50, choices=LogsType.choices)
    service = models.ForeignKey(Service, related_name='service_logs', on_delete=models.CASCADE)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        db_table = '"user_logs"'



from abc import abstractmethod

import sib_api_v3_sdk
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import EmailMessage
from django.core.mail.backends.smtp import EmailBackend
from django.template import TemplateDoesNotExist
from django.template.loader import render_to_string
from mailchimp3 import MailChimp

from drf_auth_service.common.constants import MAILCHIMP_BACKEND_NAME, SENDINBLUE_BACKEND_NAME, HTML_BACKEND_NAME
from drf_auth_service.models import ActivationCode
from drf_auth_service.settings import settings


class BaseEmailBackend:
    configs = {}
    template_reset = None
    template_confirm = None

    def __init__(self, configs: dict):
        self.configs = configs
        self.config_backend()

    @abstractmethod
    def send(self, template_id, **context):
        pass

    @abstractmethod
    def config_backend(self):
        pass

    def send_reset_password(self, user):
        token = ActivationCode.create_token(user, ActivationCode.CodeTypes.RESET_PASSWORD)
        send_reset_pass_url = f"{settings.SEND_RESET_PASSWORD_URL}{token.key}"
        self.send(self.get_template(self.template_reset), email=user.username,
                  send_reset_pass_url=send_reset_pass_url)

    def send_confirmation(self, user):
        token = ActivationCode.create_token(user, ActivationCode.CodeTypes.CONFIRM_ACCOUNT)
        send_confirmation_url = f"{settings.SEND_CONFIRMATION_URL}{token.key}"
        self.send(self.get_template(self.template_confirm), email=user.username,
                  send_confirmation_url=send_confirmation_url)

    def get_template(self, template_type):
        return self.configs[template_type]


class MailchimpBackend(BaseEmailBackend):
    name = MAILCHIMP_BACKEND_NAME
    client = None
    template_reset = 'MAILCHIMP_RESET_PASS_TEMPLATE'
    template_confirm = 'MAILCHIMP_CONFIRMATION_TEMPLATE'

    def send(self, template_id, **context):
        pass

    def config_backend(self):
        self.client = MailChimp(self.configs['MAILCHIMP_USERNAME'], self.configs['MAILCHIMP_SECRET_KEY'])


class SendInBlueBackend(BaseEmailBackend):
    name = SENDINBLUE_BACKEND_NAME
    configurations = None
    send_in_blue_instance = None
    template_reset = 'SENDINBLUE_RESET_PASS_TEMPLATE'
    template_confirm = 'SENDINBLUE_CONFIRMATION_TEMPLATE'

    def send(self, template_id, **context):
        send_smtp_email = sib_api_v3_sdk.SendSmtpEmail(to=[{"email": context['email']}],
                                                       template_id=template_id, params=context)
        self.send_in_blue_instance.send_transac_email(send_smtp_email)

    def config_backend(self):
        configuration = sib_api_v3_sdk.Configuration()
        configuration.api_key['api-key'] = self.configs['SENDINBLUE_API_KEY']
        self.send_in_blue_instance = sib_api_v3_sdk.SMTPApi(sib_api_v3_sdk.ApiClient(configuration))


class HtmlTemplateBackend(BaseEmailBackend):
    name = HTML_BACKEND_NAME
    email_backend = None
    template_reset = 'HTML_EMAIL_RESET_TEMPLATE'
    template_confirm = 'HTML_EMAIL_CONFIRM_TEMPLATE'

    def send(self, html_template, **context):
        try:
            email_html_message = render_to_string((html_template, context.get('default_template')), context)
        except TemplateDoesNotExist:
            raise ImproperlyConfigured(F"{html_template} invalid path in settings")
        self.email_backend.open()
        email = EmailMessage(context['subject'], email_html_message, self.configs['HTML_DEFAULT_FROM_EMAIL'],
                             [context['email']], [])
        self.email_backend.send_messages([email])
        self.email_backend.close()

    def config_backend(self):
        self.email_backend = EmailBackend(host=self.configs['EMAIL_HOST'],
                                          port=self.configs['EMAIL_PORT'],
                                          username=self.configs['EMAIL_USERNAME'],
                                          password=self.configs['EMAIL_PASSWORD'],
                                          use_tls=self.configs['EMAIL_USE_TLS'],
                                          fail_silently=False)

    def send_reset_password(self, user):
        token = ActivationCode.create_token(user, ActivationCode.CodeTypes.RESET_PASSWORD)
        send_reset_pass_url = f"{settings.SEND_RESET_PASSWORD_URL}{token.key}"
        self.send(self.get_template(self.template_reset), email=user.username, default_template='reset_password.html',
                  send_reset_pass_url=send_reset_pass_url, subject=self.configs['HTML_RESET_PASSWORD_SUBJECT'])

    def send_confirmation(self, user):
        token = ActivationCode.create_token(user, ActivationCode.CodeTypes.CONFIRM_ACCOUNT)
        send_confirmation_url = f"{settings.SEND_CONFIRMATION_URL}{token.key}"
        self.send(self.get_template(self.template_confirm), email=user.username,
                  send_confirmation_url=send_confirmation_url, subject=self.configs['HTML_CONFIRMATION_SUBJECT'],
                  default_template='confirm_account.html')



from abc import abstractmethod

from django.template import Context, Template
from rest_framework.exceptions import ValidationError
from twilio.base.exceptions import TwilioRestException
from twilio.rest import Client

from drf_auth_service.common.constants import TWILIO_BACKEND_NAME
from drf_auth_service.models import ActivationCode


class BasePhoneBackend:
    configs = {}
    template_reset = 'SMS_RESET_PASSWORD_MESSAGE'
    template_confirm = 'SMS_CONFIRMATION_MESSAGE'

    def __init__(self, configs: dict):
        self.configs = configs
        self.config_backend()

    @abstractmethod
    def send(self, to_phone, message, from_phone=None):
        pass

    @abstractmethod
    def config_backend(self):
        pass

    def send_reset_password(self, user):
        self.send(message=self.get_message(self.template_reset, user), to_phone=user.username)

    def send_confirmation(self, user):
        self.send(message=self.get_message(self.template_confirm, user), to_phone=user.username)

    def get_message(self, template_type, user):
        message = self.configs[template_type]
        if template_type == self.template_reset:
            code = ActivationCode.create_token(
                user, ActivationCode.CodeTypes.RESET_PASSWORD, for_phone=True
            )

        else:
            code = ActivationCode.create_token(
                user, ActivationCode.CodeTypes.CONFIRM_ACCOUNT, for_phone=True
            )

        template = Template(message)
        context = Context({"code": str(code.key)})
        return template.render(context)


class TwilioBackend(BasePhoneBackend):
    name = TWILIO_BACKEND_NAME
    client = None

    def send(self, to_phone, message, from_phone=None):
        if from_phone is None:
            from_phone = self.configs['TWILIO_FROM_NUMBER']
        try:
            self.client.messages.create(to=f"+{to_phone}", from_=from_phone, body=message)
        except TwilioRestException:
            raise ValidationError(dict(detail='Improperly configuration for twilio'))

    def config_backend(self):
        self.client = Client(self.configs['TWILIO_ACCOUNT_SID'], self.configs['TWILIO_AUTH_TOKEN'])



from rest_framework.exceptions import PermissionDenied

from drf_auth_service.models import Service


class ServiceTokenMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    @staticmethod
    def get_token(request):
        return request.headers.get('Secret-Token', None)

    def __call__(self, request):
        if request.path != '/':
            token = self.get_token(request)

            if token is None:
                raise PermissionDenied("Secret-Token is missing")

            request.service = Service.objects.filter(secret_token=token).first()

            if not request.service:
                raise PermissionDenied("Invalid Secret-Token")

        return self.get_response(request)



from drf_auth_service.common.permissions import ServiceTokenPermission

AUTHENTICATION_PERMISSIONS = dict(register=[ServiceTokenPermission],
                                  send_reset_password=[ServiceTokenPermission],
                                  reset_password_confirm=[ServiceTokenPermission],
                                  default=[ServiceTokenPermission])



from drf_auth_service.common.permissions import ServiceTokenPermission, SecretServiceTokenPermission

USER_PERMISSIONS = dict(user_confirm=[ServiceTokenPermission],
                        resend_confirmation=[SecretServiceTokenPermission],
                        destroy=[SecretServiceTokenPermission],
                        block_user=[SecretServiceTokenPermission],
                        list=[SecretServiceTokenPermission],
                        unblock_user=[SecretServiceTokenPermission],
                        default=[SecretServiceTokenPermission])



import builtins

from drf_auth_service.common.backends import BaseBackend
from drf_auth_service.common.enums import LogsType
from drf_auth_service.common.helpers import generate_code
from drf_auth_service.common.managers import EmailManager, PhoneManager
from drf_auth_service.models import Config, UserLogs
from drf_auth_service.settings import settings, User


class EmailBackend(BaseBackend):
    name = 'email'
    manager = EmailManager


class PhoneBackend(BaseBackend):
    name = 'phone_number'
    manager = PhoneManager


class PhoneCodeBackend(BaseBackend):
    name = 'phone_number_code'
    manager = PhoneManager

    def register(self):
        user, code = self.register_user()
        backend = self.manager(self.configs)
        backend.send(to_phone=user.username, message=f"Registration code: {code}")

    def register_user(self):
        user = User(username=self.request.data['username'], service=self.request.service,
                    register_type=self.request.data['register_type'])
        code = generate_code()
        user.set_password(code)
        user.save()
        UserLogs.objects.create(service=self.request.service, user=user,
                                username=user.username, type_log=LogsType.REGISTER)
        return user, code


class NicknameBackend(BaseBackend):
    name = 'nickname'

    def register(self):
        return self.get_jwt_response(self.register_user())


class RegisterManager:
    register_backend = None
    request = None
    configs = {}

    def __init__(self, register_type, request):
        self.request = request
        self.init_configs()
        self.set_backend(register_type)

    def register(self):
        return self.register_backend.register()

    def init_configs(self):
        for conf in Config.objects.filter(service=self.request.service):
            self.configs[conf.key] = getattr(builtins, conf.type)(conf.value)

    def set_backend(self, backend_name):
        for register_backend in settings.BACKENDS.REGISTER_BACKENDS:
            if getattr(register_backend, 'name') == backend_name:
                self.register_backend = register_backend(self.request, self.configs)



from rest_framework import serializers

from drf_auth_service.models import ActivationCode
from drf_auth_service.services.serializers import ServiceSerializer
from drf_auth_service.settings import User
from drf_auth_service.socials.serializers import SocialModelSerializer
from drf_auth_service.validators import validate_token_by_type


class UserBaseSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('id', 'username',)


class ReturnUserBaseSerializer(serializers.ModelSerializer):
    service = ServiceSerializer()
    user_social_identifier = SocialModelSerializer(many=True)

    class Meta:
        model = User
        fields = ('id', 'username', 'service', 'is_blocked', 'user_social_identifier', 'is_confirmed')


class UserConfirmSerializer(serializers.Serializer):
    token = serializers.CharField(required=True, allow_null=False, allow_blank=False,
                                  label="Token to confirm user")

    def validate_token(self, token):
        return validate_token_by_type(token, ActivationCode.CodeTypes.CONFIRM_ACCOUNT)


class UserIdentifierSerializer(serializers.Serializer):
    username = serializers.CharField(required=True)


class UserSetPasswordSerializer(UserIdentifierSerializer):
    password = serializers.CharField(label="Password", required=True, allow_null=False, allow_blank=False)
    confirm_password = serializers.CharField(label="Confirm password", required=True,
                                             allow_null=False, allow_blank=False)

    def validate(self, data):
        if data['password'] != data['confirm_password']:
            raise serializers.ValidationError("Passwords does not match")

        return data


class BockUserSerializer(UserIdentifierSerializer):
    reason = serializers.CharField(required=True)



from urllib.parse import urlparse

from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema, no_body
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework_simplejwt.settings import api_settings
from rest_framework_simplejwt.views import TokenViewBase

from drf_auth_service.authentication.serializers import LoginSerializer, EBSTokenRefreshSerializer, \
    ReturnRegisterSerializer, ReturnAccessTokenSerializer, ReturnJWTTokensSerializer
from drf_auth_service.common.backends import BaseBackend
from drf_auth_service.common.managers import BaseManager
from drf_auth_service.common.mixins import GenericEBSViewSet
from drf_auth_service.common.permissions import ServiceTokenPermission
from drf_auth_service.common.register_backends import RegisterManager
from drf_auth_service.models import ActivationCode
from drf_auth_service.settings import settings

jwt_response = openapi.Response('Respond with jwt access&refresh token', ReturnRegisterSerializer)
refresh_response = openapi.Response('Respond with jwt access token, ', ReturnAccessTokenSerializer)


class AuthenticationViewSet(GenericEBSViewSet):
    serializer_create_class = settings.SERIALIZERS.REGISTER_SERIALIZER
    serializer_class = settings.SERIALIZERS.REGISTER_RETURN_SERIALIZER
    permission_classes_by_action = settings.PERMISSIONS.AUTHENTICATION_PERMISSIONS

    @action(detail=False, methods=['POST'])
    def register(self, request, *args, **kwargs):
        api_settings.SIGNING_KEY = request.service.secret_token
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        register_manager = RegisterManager(register_type=serializer.validated_data['register_type'], request=request)
        response = register_manager.register()
        return Response(response)

    @action(detail=False, methods=['POST'], url_path='restore-request',
            serializer_class=settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER,
            serializer_create_class=settings.SERIALIZERS.SEND_RESET_PASSWORD_SERIALIZER)
    def send_reset_password(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        manager = BaseManager.load_manager(serializer.validated_data['user'], configs=None, request=request)
        manager.send_reset_password(serializer.validated_data['user'])
        return Response(self.get_serializer(dict(message='Reset password was sent successfully')).data)

    @action(detail=False, methods=['POST'], url_path='restore-confirm',
            serializer_class=ReturnJWTTokensSerializer,
            serializer_create_class=settings.SERIALIZERS.RESET_PASSWORD_CONFIRMATION_SERIALIZER)
    def reset_password_confirm(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        password = serializer.validated_data['password']

        reset_password_token = serializer.validated_data['token']
        reset_password_token.user.set_password(password)
        reset_password_token.user.save()

        ActivationCode.make_user_active(reset_password_token.user)
        return Response(BaseBackend.get_jwt_response(reset_password_token.user))

    @swagger_auto_schema(method='GET', request_body=no_body, responses={})
    @action(detail=False, methods=['GET'])
    def logout(self, request, *args, **kwargs):
        response = Response(status=status.HTTP_200_OK)
        response.set_cookie(
            settings.COOKIE_KEY,
            "",  # Clean the token
            max_age=settings.REFRESH_TOKEN_LIFETIME.total_seconds(),
            domain=urlparse(settings.DOMAIN_ADDRESS).netloc if settings.DOMAIN_ADDRESS else None,
            httponly=True,
            secure=True
        )
        response.cookies[settings.COOKIE_KEY]['samesite'] = 'None'
        return response


class LoginViewSet(TokenViewBase):
    serializer_class = LoginSerializer
    permission_classes = (ServiceTokenPermission,)

    @swagger_auto_schema(request_body=LoginSerializer, responses={200: jwt_response})
    def post(self, request, *args, **kwargs):
        api_settings.SIGNING_KEY = request.service.secret_token
        response = super(LoginViewSet, self).post(request, *args, **kwargs)
        response.set_cookie(
            settings.COOKIE_KEY,
            response.data['refresh'],
            max_age=settings.REFRESH_TOKEN_LIFETIME.total_seconds(),
            domain=urlparse(settings.DOMAIN_ADDRESS).netloc if settings.DOMAIN_ADDRESS else None,
            httponly=True,
            secure=True
        )
        response.cookies[settings.COOKIE_KEY]['samesite'] = 'None'
        return response


class EBSTokenRefreshView(TokenViewBase):
    serializer_class = EBSTokenRefreshSerializer
    permission_classes = (ServiceTokenPermission,)

    @swagger_auto_schema(request_body=EBSTokenRefreshSerializer, responses={200: refresh_response})
    def post(self, request, *args, **kwargs):
        api_settings.SIGNING_KEY = request.service.secret_token
        return super().post(request, *args, **kwargs)



from django.db import transaction
from rest_framework_simplejwt.authentication import JWTAuthentication
from rest_framework_simplejwt.exceptions import AuthenticationFailed, InvalidToken
from rest_framework_simplejwt.settings import api_settings
from rest_framework_simplejwt.tokens import RefreshToken

from drf_auth_service.common.enums import LogsType
from drf_auth_service.models import UserLogs
from drf_auth_service.settings import User
from drf_auth_service.settings import settings


class BaseBackend:
    configs = {}
    manager = None
    name = None

    def __init__(self, request, configs):
        self.request = request
        self.configs = configs

    def register(self):
        with transaction.atomic():
            user = self.register_user()
            if self.configs['SEND_CONFIRMATION']:
                return self.send_confirmation(user)

        return self.get_jwt_response(user)

    def register_user(self):
        user = User(username=self.request.data['username'], service=self.request.service,
                    register_type=self.name)
        user.set_password(self.request.data['password'])
        user.save()
        UserLogs.objects.create(service=self.request.service, user=user,
                                username=user.username, type_log=LogsType.REGISTER)
        return user

    def send_confirmation(self, user):
        backend = self.manager(self.configs)
        backend.send_confirmation(user)
        return settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER(dict(message="Confirmation was sent successfully")).data

    @staticmethod
    def get_jwt_response(user):
        refresh = RefreshToken.for_user(user)
        refresh['username'] = user.username
        refresh['user_id'] = user.id

        return dict(refresh_token=str(refresh), access_token=str(refresh.access_token))


class SSOAuthentication(JWTAuthentication):
    request = None

    def authenticate(self, request):
        self.request = request
        return super(SSOAuthentication, self).authenticate(request)

    def get_user(self, validated_token):
        try:
            user_id = validated_token[api_settings.USER_ID_CLAIM]
        except KeyError:
            raise InvalidToken('Token contained no recognizable user identification')

        try:
            user = User.objects.get(**{
                api_settings.USER_ID_FIELD: user_id,
                'service': getattr(self.request, 'service', None)
            })
        except User.DoesNotExist:
            raise AuthenticationFailed('User not found', code='user_not_found')

        if user.is_blocked:
            raise AuthenticationFailed('User is blocked', code='user_blocked')

        if not user.is_confirmed:
            raise AuthenticationFailed('User is not confirmed', code='user_not_confirmed')



import binascii
import os
import random


def generate_token(min_length=30, max_length=40):
    length = random.randint(min_length, max_length)

    return binascii.hexlify(
        os.urandom(max_length)
    ).decode()[0:length]


def generate_code(length=6):
    range_start = 10 ** (length - 1)
    range_end = (10 ** length) - 1
    return random.randint(range_start, range_end)



from rest_framework.reverse import reverse
from rest_framework.test import APITestCase


class AuthorizationViewTest(APITestCase):
    def setUp(self) -> None:
        self.base_url = reverse("authentication")

    def test_test(self):
        print('intra')


import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

DEBUG = True

SECRET_KEY = '༼ つ ◕_◕ ༽つ'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'precept',
        'USER': 'precept',
        'PASSWORD': 'precept',
        'HOST': 'localhost',
        'PORT': '5432',
    }
}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'drf_auth_service'
]

AUTH_USER_MODEL = 'drf_auth_service.SSOUser'
MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    # 'apps.common.middlewares.ApiMiddleware',
]

ROOT_URLCONF = 'urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

REST_FRAMEWORK = {
    'DATETIME_FORMAT': "%Y-%m-%dT%H:%M:%SZ",
    'PAGE_SIZE': 50,
    'DEFAULT_AUTHENTICATION_CLASSES': ['drf_auth_service.common.backends.SSOAuthentication'],
    'DEFAULT_PERMISSION_CLASSES': ('rest_framework.permissions.AllowAny',),
    'DEFAULT_RENDERER_CLASSES': ('rest_framework.renderers.JSONRenderer',),
}


FIXTURE_DIRS = (
    'fixtures/',
)

# SSO = {
#     "SERIALIZERS": {
#         "REGISTER_SERIALIZER": "apps.authentication.serializers.RegisterSerializer",
#         "SEND_RESET_PASSWORD_SERIALIZER": "apps.authentication.serializers.ResetPasswordSerializer"
#     },
#     "VIEWS": {
#         "USER_VIEWS": "apps.users.views.CustomUserViewSet",
#     },
#     "BACKENDS": {
#         "SMS_BACKEND": "apps.common.backends.CustomPhoneProvider",
#         "REGISTER_BACKENDS": [
#             'apps.common.backends.CustomRegisterBackend',
#         ]
#     }
# }



from rest_framework.exceptions import PermissionDenied
from rest_framework.permissions import BasePermission

from drf_auth_service.models import Service


class ServiceTokenPermission(BasePermission):

    @staticmethod
    def get_token(request):
        return request.headers.get('Service-Token', None)

    def has_permission(self, request, view):
        token = self.get_token(request)

        if token is None:
            raise PermissionDenied("Service-Token is missing")

        request.service = Service.objects.filter(public_token=token).first()

        if not request.service:
            raise PermissionDenied("Invalid Service-Token")

        return True


class SecretServiceTokenPermission(BasePermission):

    @staticmethod
    def get_tokens(request):
        return request.headers.get('Service-Token', None), request.headers.get('Secret-Token', None)

    def has_permission(self, request, view):
        service_token, secret_token = self.get_tokens(request)

        if service_token is None:
            raise PermissionDenied("Service-Token is missing")

        if secret_token is None:
            raise PermissionDenied("Secret-Token is missing")

        request.service = Service.objects.filter(public_token=service_token, secret_token=secret_token).first()

        if not request.service:
            raise PermissionDenied("Invalid Service-Token")

        return True



from django.db import models
from django.utils.translation import ugettext as _


class RegisterType(models.TextChoices):
    EMAIL = 'email', _('Email')
    PHONE_NUMBER = 'phone_number', _('Phone number')
    PHONE_NUMBER_CODE = 'phone_number_code', _('Phone number code')
    NICKNAME = 'nickname', _('Nickname')


class SocialTypes(models.TextChoices):
    Facebook = 'facebook', _('Facebook')
    Google = 'google-oauth2', _('Google')
    Apple = 'apple-id', _('Apple')


class LogsType(models.TextChoices):
    LOGIN_BY_CREDENTIALS = "login_by_credentials",
    REFRESH_TOKEN = "refresh_token",
    LOGOUT = "logout",
    REGISTER = "register",
    TOKEN_SENT = 'token_sent',
    TOKEN_CONFIRMED = 'token_confirmed',



from django.db import models
from django.utils.translation import gettext as _


class DeviceTypes(models.TextChoices):
    ANDROID = 'android', _('Android')
    IOS = 'ios', _('Ios')
    WEB = 'web', _('Web')



from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema
from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
from six import text_type
from social_django.utils import load_strategy

from drf_auth_service.common.permissions import ServiceTokenPermission
from drf_auth_service.models import SSOUser
from drf_auth_service.settings import settings
from drf_auth_service.socials.helpers import get_user_data_from_provider
from drf_auth_service.socials.serializers import SocialSerializer, ReturnSocialSerializer

oauth_response = openapi.Response('Respond with jwt secret token if user exists', ReturnSocialSerializer)


@swagger_auto_schema(method='POST', request_body=SocialSerializer, responses={200: oauth_response})
@api_view(['POST'])
@permission_classes([ServiceTokenPermission])
def social_login(self, request):
    """Authenticate user through the provider(facebook/google/apple) and access_token"""

    user_exist = False
    serializer = self.serializer_class(data=request.data)
    serializer.is_valid(raise_exception=True)
    provider = serializer.data.get('provider', None)
    strategy = load_strategy(request)
    user_data = get_user_data_from_provider(strategy, provider, serializer.data.get('social_token'),
                                            serializer.data.get('device_type'))
    response_data = dict(exists=user_exist, data=user_data)

    if provider == settings.ENUMS.SOCIAL_TYPES.Facebook:
        response_data['data'].update(dict(picture=user_data['picture']['data'].get('url', None)))
        user_exist = SSOUser.objects.filter(user_social_identifier__identifier=user_data.get('id'))

    elif provider == settings.ENUMS.SOCIAL_TYPES.Google:
        user_exist = SSOUser.objects.filter(user_social_identifier__identifier=user_data.get('sub'))
        response_data['data'] = dict(id=user_data.get('sub'), first_name=user_data.get('given_name', None),
                                     last_name=user_data.get('family_name', None),
                                     picture=user_data.get('picture', None), email=user_data.get('email', None))

    elif provider == settings.ENUMS.SOCIAL_TYPES.Apple:
        raise NotImplementedError()

    if user_exist and user_exist.exists():
        refresh = TokenObtainPairSerializer.get_token(user_exist.first())
        response_data['exists'] = user_exist.exists()
        response_data['auth'] = dict(pwd_token=user_exist.first().pwd_token, access=text_type(refresh.access_token))

    return Response(response_data)



from rest_framework.decorators import action
from rest_framework.response import Response

from drf_auth_service.common.mixins import EBSModelViewSet
from drf_auth_service.models import Service, Config
from drf_auth_service.settings import settings


class ServiceViewSet(EBSModelViewSet):
    serializer_class = settings.SERIALIZERS.SERVICE_SERIALIZER
    queryset = Service.objects.all()

    @action(detail=True, methods=['GET', 'PATCH'])
    def configs(self, request, *args, **kwargs):
        service = self.get_object()

        if request.method == 'GET':
            configs = {config['key']: config['value'] for config in service.configs.all().values('key', 'value')}
            return Response(configs)

        else:
            configs = Config.objects.filter(service=service, key__in=request.data.keys())

            for config in configs:
                config.value = request.data[config.key]

            Config.objects.bulk_update(configs, ['value'])

            return Response(request.data)



# Generated by Django 3.1.3 on 2020-11-09 12:54

import django.utils.timezone
from django.db import migrations, models

import drf_auth_service.common.helpers


class Migration(migrations.Migration):
    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SSOUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('username', models.CharField(max_length=200, unique=True)),
                ('register_type', models.CharField(choices=[('email', 'Email'), ('phone_number', 'Phone number'),
                                                            ('phone_number_code', 'Phone number code'),
                                                            ('nickname', 'Nickname')], default='email',
                                                   max_length=100)),
            ],
            options={
                'db_table': '"users"',
                'abstract': False,
                'swappable': 'AUTH_USER_MODEL',
            },
        ),
        migrations.CreateModel(
            name='ActivationCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('key', models.CharField(db_index=True, max_length=64, unique=True)),
                ('key_type', models.CharField(
                    choices=[('reset_password', 'Reset password'), ('confirm_account', 'Confirm account')],
                    default='reset_password', max_length=30)),
            ],
            options={
                'db_table': '"activation_codes"',
            },
        ),
        migrations.CreateModel(
            name='Config',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('key', models.CharField(max_length=200)),
                ('type', models.CharField(max_length=200)),
                ('value', models.TextField()),
            ],
            options={
                'db_table': '"configs"',
            },
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=300)),
                ('description', models.CharField(max_length=300)),
            ],
            options={
                'db_table': '"roles"',
            },
        ),
        migrations.CreateModel(
            name='Scope',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.TextField()),
                ('name', models.CharField(max_length=50)),
                ('code', models.CharField(max_length=200)),
            ],
            options={
                'db_table': '"scopes"',
            },
        ),
        migrations.CreateModel(
            name='Service',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('secret_token',
                 models.CharField(default=drf_auth_service.common.helpers.generate_token, max_length=200, unique=True)),
                ('public_token',
                 models.CharField(default=drf_auth_service.common.helpers.generate_token, max_length=200, unique=True)),
            ],
            options={
                'db_table': '"services"',
            },
        ),
        migrations.CreateModel(
            name='Social',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('label', models.CharField(max_length=100)),
                ('identifier', models.CharField(max_length=100)),
                ('social_type', models.CharField(
                    choices=[('facebook', 'Facebook'), ('google-oauth2', 'Google'), ('apple-id', 'Apple')],
                    max_length=20)),
            ],
            options={
                'db_table': '"socials"',
            },
        ),
        migrations.CreateModel(
            name='UserBlock',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('ip', models.CharField(max_length=50, null=True)),
                ('reason', models.CharField(max_length=254, null=True)),
            ],
            options={
                'db_table': '"user_blocks"',
            },
        ),
        migrations.CreateModel(
            name='UserLogs',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(max_length=200)),
                ('type_log', models.CharField(
                    choices=[('login_by_credentials', 'Login By Credentials'), ('refresh_token', 'Refresh Token'),
                             ('logout', 'Logout'), ('register', 'Register'), ('token_sent', 'Token Sent'),
                             ('token_confirmed', 'Token Confirmed')], max_length=50)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('service', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='service_logs',
                                              to='drf_auth_service.service')),
            ],
            options={
                'db_table': '"user_logs"',
            },
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-20 13:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('drf_auth_service', '0002_auto_20201109_1254'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ssouser',
            name='service',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='service_users', to='drf_auth_service.service'),
        ),
    ]



import builtins

from django.core.exceptions import ImproperlyConfigured
from django.utils.module_loading import import_string

from drf_auth_service.common.email_backends import BaseEmailBackend
from drf_auth_service.common.enums import RegisterType
from drf_auth_service.common.sms_backends import BasePhoneBackend
from drf_auth_service.models import Config


class BaseManager:
    backend = None
    backend_type = None
    base_backend = None

    def __init__(self, configs, request=None):
        if configs is None and request:
            configs = self.get_configs(request)
        elif configs is None:
            raise Warning('Can not be both configs and request None')
        try:
            self.backend = import_string(configs[self.backend_type])(configs)
            if not isinstance(self.backend, self.base_backend):
                raise ImproperlyConfigured(
                    f"{configs[self.backend_type]} is not instance of BaseEmailBackend"
                )
        except ImportError:
            raise ImproperlyConfigured(
                'Invalid path for backend'
            )

    def send_confirmation(self, user):
        self.backend.send_confirmation(user)

    def send_reset_password(self, user):
        self.backend.send_reset_password(user)

    @staticmethod
    def get_configs(request):
        configs = {}
        for conf in Config.objects.filter(service=request.service):
            configs[conf.key] = getattr(builtins, conf.type)(conf.value)
        return configs

    @staticmethod
    def load_manager(user, configs, request=None):

        if RegisterType.EMAIL in user.register_type:
            return EmailManager(configs, request)
        else:
            return PhoneManager(configs, request)


class PhoneManager(BaseManager):
    backend = None
    backend_type = 'SMS_BACKEND'
    base_backend = BasePhoneBackend

    def send(self, to_phone, message, from_phone=None):
        self.backend.send(to_phone, message, from_phone)


class EmailManager(BaseManager):
    backend = None
    backend_type = 'EMAIL_BACKEND'
    base_backend = BaseEmailBackend

    def send(self, email_to, **context):
        self.backend.send(email=email_to, **context)



from datetime import timedelta

from django.apps import apps
from django.conf import settings as django_settings
from django.core.exceptions import ImproperlyConfigured
from django.core.signals import setting_changed
from django.utils.functional import LazyObject
from django.utils.module_loading import import_string

from drf_auth_service.common.constants import SENDINBLUE_BACKEND_NAME, MAILCHIMP_BACKEND_NAME, TWILIO_BACKEND_NAME
from drf_auth_service.common.enums import RegisterType

auth_user = getattr(django_settings, 'AUTH_USER_MODEL', None) or 'drf_auth_service.SSOUser'
auth_module, user_model = auth_user.rsplit(".", 1)

User = apps.get_model(auth_module, user_model)
SSO_SETTINGS_NAMESPACE = "DRF_AUTH_SERVICE"

EMAIL_HOST = getattr(django_settings, 'EMAIL_HOST')
EMAIL_PORT = getattr(django_settings, 'EMAIL_PORT')
EMAIL_USERNAME = getattr(django_settings, 'EMAIL_HOST_USER')
EMAIL_PASSWORD = getattr(django_settings, 'EMAIL_HOST_PASSWORD')
EMAIL_USE_TLS = getattr(django_settings, 'EMAIL_USE_TLS')


class ObjDict(dict):
    def __getattribute__(self, item):
        try:
            value = self[item]
            if isinstance(value, str):
                value = import_string(value)
            elif isinstance(value, (list, tuple)):
                value = [import_string(val) if isinstance(val, str) else val for val in value]
            self[item] = value
        except KeyError:
            value = super(ObjDict, self).__getattribute__(item)

        return value

    def __set_name__(self, owner, name):
        self.__name__ = 'dict'


default_sso_settings = {
    "WORK_MODE": "standalone",
    "COOKIE_KEY": "sso_key",
    "DOMAIN_ADDRESS": None,
    "REFRESH_TOKEN_LIFETIME": timedelta(days=30),
    "SENDINBLUE_RESET_PASS_TEMPLATE": None,
    "SENDINBLUE_CONFIRMATION_TEMPLATE": None,
    "SENDINBLUE_API_KEY": None,
    "MAILCHIMP_RESET_PASS_TEMPLATE": None,
    "MAILCHIMP_CONFIRMATION_TEMPLATE": None,
    "MAILCHIMP_USERNAME": None,
    "MAILCHIMP_SECRET_KEY": None,
    "SEND_CONFIRMATION": True,
    "RESET_PASSWORD_EXPIRE": 24,
    "SEND_RESET_PASSWORD_URL": 'https://example.com/reset-password/',
    "SEND_CONFIRMATION_URL": 'https://example.com/confirm-email/',
    "TWILIO_ACCOUNT_SID": "AC6b3fc1744aa63d35235a4fb01241e71a",
    "TWILIO_AUTH_TOKEN": "a3e0f68829427c69dbb94cb4e95ff638",
    "TWILIO_FROM_NUMBER": None,
    "SMS_CONFIRMATION_MESSAGE": "Confirmation code {{ code }}",
    "SMS_RESET_PASSWORD_MESSAGE": "Reset password code {{ code }}",
    "REGISTER_TYPES": list(dict(RegisterType.choices).keys()),
    "EMAIL_HOST": EMAIL_HOST,
    "EMAIL_PORT": EMAIL_PORT,
    "EMAIL_USERNAME": EMAIL_USERNAME,
    "EMAIL_PASSWORD": EMAIL_PASSWORD,
    "EMAIL_USE_TLS": EMAIL_USE_TLS,
    "HTML_RESET_PASSWORD_SUBJECT": "Reset password",
    "HTML_DEFAULT_FROM_EMAIL": EMAIL_USERNAME,
    "HTML_CONFIRMATION_SUBJECT": "Confirm account",
    "HTML_EMAIL_RESET_TEMPLATE": None,
    "HTML_EMAIL_CONFIRM_TEMPLATE": None,
    "SERIALIZERS": ObjDict({
        "USER_RETURN_SERIALIZER": "drf_auth_service.users.serializers.ReturnUserBaseSerializer",
        "USER_SERIALIZER": "drf_auth_service.users.serializers.UserBaseSerializer",
        "BLOCK_USER_SERIALIZER": "drf_auth_service.users.serializers.BockUserSerializer",
        "USER_CONFIRM_SERIALIZER": "drf_auth_service.users.serializers.UserConfirmSerializer",
        "USER_IDENTIFIER": "drf_auth_service.users.serializers.UserIdentifierSerializer",
        "SOCIAL_RETURN_SERIALIZER": "drf_auth_service.socials.serializers.ReturnSocialSerializer",
        "SOCIAL_SERIALIZER": "drf_auth_service.socials.serializers.SocialSerializer",
        "SERVICE_SERIALIZER": "drf_auth_service.services.serializers.ServiceSerializer",
        "REGISTER_SERIALIZER": "drf_auth_service.authentication.serializers.RegisterSerializer",
        "REGISTER_RETURN_SERIALIZER": "drf_auth_service.authentication.serializers.ReturnRegisterSerializer",
        "SEND_RESET_PASSWORD_SERIALIZER": "drf_auth_service.authentication.serializers.SendResetPasswordSerializer",
        "RESET_PASSWORD_CONFIRMATION_SERIALIZER": "drf_auth_service.authentication.serializers.ResetPasswordConfirmSerializer",
        "RETURN_SUCCESS_SERIALIZER": "drf_auth_service.authentication.serializers.ReturnSuccessSerializer",
        "SET_PASSWORD_SERIALIZER": "drf_auth_service.users.serializers.UserSetPasswordSerializer",
    }),
    "VIEWS": ObjDict({
        "USER_VIEWS": "drf_auth_service.users.views.UserViewSet",
        "SERVICE_VIEWS": "drf_auth_service.services.views.ServiceViewSet",
        "AUTHENTICATION_VIEWS": "drf_auth_service.authentication.views.AuthenticationViewSet",
    }),
    "PERMISSIONS": ObjDict({
        "USER_PERMISSIONS": "drf_auth_service.users.permissions.USER_PERMISSIONS",
        "AUTHENTICATION_PERMISSIONS": "drf_auth_service.authentication.permissions.AUTHENTICATION_PERMISSIONS"
    }),
    "BACKENDS": ObjDict({
        "SMS_BACKEND": "drf_auth_service.common.sms_backends.TwilioBackend",
        "EMAIL_BACKEND": "drf_auth_service.common.email_backends.HtmlTemplateBackend",
        "REGISTER_BACKENDS": [
            'drf_auth_service.common.register_backends.EmailBackend',
            'drf_auth_service.common.register_backends.PhoneBackend',
            'drf_auth_service.common.register_backends.PhoneCodeBackend',
            'drf_auth_service.common.register_backends.NicknameBackend',
        ]
    }),
    "ENUMS": ObjDict({
        "REGISTER_TYPES": "drf_auth_service.common.enums.RegisterType",
        "SOCIAL_TYPES": "drf_auth_service.common.enums.SocialTypes",
    })
}

MAILCHIMP_REQUIRED_CONFIGS = ["MAILCHIMP_SECRET_KEY", "MAILCHIMP_RESET_PASS_TEMPLATE",
                              "MAILCHIMP_CONFIRMATION_TEMPLATE", "MAILCHIMP_SECRET_KEY"]

SENDINBLUE_REQUIRED_CONFIGS = ["SENDINBLUE_API_KEY", "SENDINBLUE_RESET_PASS_TEMPLATE",
                               "SENDINBLUE_CONFIRMATION_TEMPLATE"]

HTML_EMAIL_REQUIRED_CONFIGS = ["HTML_RESET_PASSWORD_SUBJECT", "HTML_CONFIRMATION_SUBJECT", "EMAIL_HOST", "EMAIL_PORT",
                               "EMAIL_USERNAME", "EMAIL_PASSWORD", "EMAIL_USE_TLS", "HTML_EMAIL_RESET_TEMPLATE",
                               "HTML_EMAIL_CONFIRM_TEMPLATE", "HTML_DEFAULT_FROM_EMAIL"]

TWILIO_REQUIRED_CONFIGS = ["TWILIO_ACCOUNT_SID", "TWILIO_AUTH_TOKEN", "TWILIO_FROM_NUMBER"]
SETTINGS_TO_IMPORT_IN_DB = ["BACKENDS", "REGISTER_TYPES", "SEND_CONFIRMATION", "SMS_RESET_PASSWORD_MESSAGE",
                            "SMS_CONFIRMATION_MESSAGE", ] + SENDINBLUE_REQUIRED_CONFIGS \
                           + MAILCHIMP_REQUIRED_CONFIGS + HTML_EMAIL_REQUIRED_CONFIGS + TWILIO_REQUIRED_CONFIGS

PHONE_NUMBER_BACKEND_CASE = []


class Settings:
    def __init__(self, default_sso_settings, explicit_overriden_settings: dict = None):
        if explicit_overriden_settings is None:
            explicit_overriden_settings = {}

        overriden_settings = (getattr(django_settings, SSO_SETTINGS_NAMESPACE, {}) or explicit_overriden_settings)
        self._load_default_sso_settings()
        self._override_settings(overriden_settings)
        self._validate_configs()
        self._add_custom_schema_to_swagger()
        try:
            self._init_settings_to_db()
        except:
            pass

    def _add_custom_schema_to_swagger(self):
        try:
            from drf_yasg import app_settings
            swagger_schema = getattr(getattr(django_settings, 'SWAGGER_SETTINGS', {}), 'DEFAULT_AUTO_SCHEMA_CLASS',
                                     None)
            if swagger_schema == app_settings.SWAGGER_DEFAULTS['DEFAULT_AUTO_SCHEMA_CLASS'] or swagger_schema is None:
                app_settings.SWAGGER_DEFAULTS[
                    'DEFAULT_AUTO_SCHEMA_CLASS'] = 'drf_auth_service.common.mixins.CustomAutoSchema'

            swagger_schema = getattr(getattr(django_settings, 'SWAGGER_SETTINGS', {}), 'SECURITY_DEFINITIONS',
                                     None)
            if swagger_schema == app_settings.SWAGGER_DEFAULTS['SECURITY_DEFINITIONS'] or swagger_schema is None:
                app_settings.SWAGGER_DEFAULTS['SECURITY_DEFINITIONS'] = {
                    'Bearer': {
                        'type': 'apiKey',
                        'name': 'Authorization',
                        'in': 'header'
                    },
                    'Service-Token': {
                        'type': 'apiKey',
                        'name': 'Service-Token',
                        'in': 'header'
                    },
                    'Secret-Token': {
                        'type': 'apiKey',
                        'name': 'Secret-Token',
                        'in': 'header'
                    }
                }
        except ModuleNotFoundError:
            pass

    def _load_default_sso_settings(self):
        for setting_name, setting_value in default_sso_settings.items():
            if setting_name.isupper():
                setattr(self, setting_name, setting_value)

    def _override_settings(self, overriden_settings: dict):
        for setting_name, setting_value in overriden_settings.items():
            value = setting_value
            if isinstance(setting_value, dict):
                value = getattr(self, setting_name, {})
                value.update(ObjDict(setting_value))
            setattr(self, setting_name, value)

    def _init_settings_to_db(self):
        from drf_auth_service.models import Service, Config

        if getattr(self, 'WORK_MODE') == 'standalone' and Service.objects.count() == 0:
            service = Service.objects.create(name='Default configs')
            configs = []

            for key, val in self.__dict__.items():
                if key in SETTINGS_TO_IMPORT_IN_DB:
                    if type(val) == ObjDict or type(val) == dict:

                        for dict_key, dict_value in val.items():
                            if not type(dict_value) == list:
                                configs.append(Config(type='str', value=dict_value, key=dict_key,
                                                      service=service))
                    elif type(val) == list:
                        value = ''
                        for list_value in val:
                            value += f"{list_value},"

                        configs.append(Config(type='str', value=value.rstrip(','), key=key,
                                              service=service))
                    else:
                        configs.append(Config(type=type(val).__name__ if type(val) is not None else 'str',
                                              value=str(val), key=key, service=service))

            Config.objects.bulk_create(configs)

    def _validate_configs(self):

        if import_string(getattr(self, 'BACKENDS')['EMAIL_BACKEND']).name == SENDINBLUE_BACKEND_NAME:
            for required_conf in SENDINBLUE_REQUIRED_CONFIGS:
                if getattr(self, required_conf, '') == '':
                    raise ImproperlyConfigured(f"{required_conf} is required for {SENDINBLUE_BACKEND_NAME}")

        if import_string(getattr(self, 'BACKENDS')['EMAIL_BACKEND']).name == MAILCHIMP_BACKEND_NAME:
            for required_conf in MAILCHIMP_REQUIRED_CONFIGS:
                if getattr(self, required_conf, '') == '':
                    raise ImproperlyConfigured(f"{required_conf} is required for {MAILCHIMP_BACKEND_NAME}")

        if import_string(getattr(self, 'BACKENDS')['SMS_BACKEND']).name == TWILIO_BACKEND_NAME:
            for required_conf in TWILIO_REQUIRED_CONFIGS:
                if getattr(self, required_conf, '') == '':
                    raise ImproperlyConfigured(f"{required_conf} is required for {TWILIO_BACKEND_NAME}")


class LazySettings(LazyObject):
    def _setup(self, explicit_overriden_settings=None):
        self._wrapped = Settings(default_sso_settings, explicit_overriden_settings)


settings = LazySettings()


def reload_sso_settings(*args, **kwargs):
    global settings
    setting, value = kwargs["setting"], kwargs["value"]
    if setting == SSO_SETTINGS_NAMESPACE:
        settings._setup(explicit_overriden_settings=value)


setting_changed.connect(reload_sso_settings)



from rest_framework import mixins, status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.settings import api_settings
from rest_framework.viewsets import GenericViewSet


class CreateEBSModelMixin:

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        model_obj = self.perform_create(serializer)
        serializer_display = self.get_serializer(model_obj)
        headers = self.get_success_headers(serializer_display.data)
        return Response(serializer_display.data, status=status.HTTP_201_CREATED, headers=headers)

    def perform_create(self, serializer):
        return serializer.save()

    def get_success_headers(self, data):
        try:
            return {'Location': str(data[api_settings.URL_FIELD_NAME])}
        except (TypeError, KeyError):
            return {}


class UpdateEBSModelMixin:

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer_create(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        model_obj = self.perform_update(serializer)
        serializer_display = self.get_serializer(model_obj)

        if getattr(instance, '_prefetched_objects_cache', None):
            instance._prefetched_objects_cache = {}

        return Response(serializer_display.data)

    def perform_update(self, serializer):
        return serializer.save()

    def partial_update(self, request, *args, **kwargs):
        kwargs['partial'] = True
        return self.update(request, *args, **kwargs)


class GenericEBSViewSet(GenericViewSet):
    serializer_create_class = None
    permission_classes_by_action = {"default": [AllowAny]}
    action_serializers = {}
    permission_classes = None
    serializer_query_class = None

    def get_serializer_class(self):
        if self.action in self.action_serializers:
            return self.action_serializers[self.action]

        return super().get_serializer_class()

    def get_permissions(self):
        try:
            # return permission_classes depending on `action`
            return [permission() for permission in self.permission_classes_by_action[self.action]]
        except KeyError:
            # action is not set return default permission_classes

            if self.permission_classes:
                return [permission() for permission in self.permission_classes]
            return [permission() for permission in self.permission_classes_by_action['default']]

    def get_serializer_create(self, *args, **kwargs):
        serializer_class = self.get_serializer_create_class()
        kwargs['context'] = self.get_serializer_context()
        return serializer_class(*args, **kwargs)

    def get_query_serializer(self):
        if self.action == ['retrieve', 'post', 'patch']:
            return None
        return self.serializer_query_class

    def get_serializer_create_class(self):
        return self.serializer_create_class if self.serializer_create_class is not None else self.serializer_class


class EBSModelViewSet(
    mixins.RetrieveModelMixin,
    mixins.DestroyModelMixin,
    mixins.ListModelMixin,
    CreateEBSModelMixin,
    UpdateEBSModelMixin,
    GenericEBSViewSet
):
    pass


try:
    from drf_yasg.inspectors import SwaggerAutoSchema
    from drf_yasg.inspectors.base import call_view_method
    from drf_yasg.utils import no_body


    class CustomAutoSchema(SwaggerAutoSchema):
        def get_view_response_serializer(self):
            return call_view_method(self.view, 'get_serializer')

        def get_view_serializer(self):
            return call_view_method(self.view, 'get_serializer_create')

        def get_view_query_serializer(self):
            return call_view_method(self.view, 'get_query_serializer')

        def get_default_response_serializer(self):
            body_override = self._get_request_body_override()
            if body_override and body_override is not no_body:
                return body_override
            if body_override is no_body:
                return None
            return self.get_view_response_serializer()

        def get_query_serializer(self):
            if self.overrides.get('query_serializer', None) is None:
                self.overrides['query_serializer'] = self.get_view_query_serializer()
            return super(CustomAutoSchema, self).get_query_serializer()
except ModuleNotFoundError:
    pass



from drf_yasg import openapi
from rest_framework.decorators import action
from rest_framework.exceptions import ValidationError
from rest_framework.mixins import ListModelMixin
from rest_framework.response import Response
from rest_framework_simplejwt.settings import api_settings

from drf_auth_service.authentication.serializers import ReturnJWTTokensSerializer, ReturnSuccessSerializer
from drf_auth_service.common.backends import BaseBackend
from drf_auth_service.common.managers import BaseManager
from drf_auth_service.common.mixins import GenericEBSViewSet
from drf_auth_service.models import ActivationCode, UserBlock
from drf_auth_service.settings import User
from drf_auth_service.settings import settings

delete_user = openapi.Response('Endpoint to delete user by username', ReturnSuccessSerializer)


class UserViewSet(
    GenericEBSViewSet,
    ListModelMixin
):
    queryset = User.objects.all()
    serializer_class = settings.SERIALIZERS.USER_RETURN_SERIALIZER
    serializer_create_class = settings.SERIALIZERS.USER_SERIALIZER
    permission_classes_by_action = settings.PERMISSIONS.USER_PERMISSIONS

    @action(detail=False, methods=['POST'], serializer_create_class=settings.SERIALIZERS.USER_CONFIRM_SERIALIZER,
            serializer_class=ReturnJWTTokensSerializer, url_path='confirm')
    def user_confirm(self, request, *args, **kwargs):
        api_settings.SIGNING_KEY = request.service.secret_token
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        reset_password_token = serializer.validated_data['token']
        ActivationCode.make_user_active(reset_password_token.user)
        return Response(BaseBackend.get_jwt_response(reset_password_token.user))

    @action(detail=False, methods=['POST'], url_path='resend-confirmation',
            serializer_class=settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER,
            serializer_create_class=settings.SERIALIZERS.USER_IDENTIFIER)
    def resend_confirmation(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        user = self.get_user_by_username(request.service, serializer.validated_data['username'])
        manager = BaseManager.load_manager(user, configs=None, request=request)
        manager.send_confirmation(user)
        return Response(self.get_serializer(dict(message='Confirmation was resent successfully')).data)

    @action(detail=False, methods=['POST'], url_path='block',
            serializer_class=settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER,
            serializer_create_class=settings.SERIALIZERS.BLOCK_USER_SERIALIZER)
    def block_user(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        user = self.get_user_by_username(request.service, serializer.validated_data['username'])
        UserBlock.objects.create(reason=serializer.validated_data['reason'], user=user)
        return Response(
            self.get_serializer(dict(message=f"User {user.username} was successfully blocked")).data
        )

    @action(detail=False, methods=['POST'], url_path='unblock',
            serializer_create_class=settings.SERIALIZERS.USER_IDENTIFIER,
            serializer_class=settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER)
    def unblock_user(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        user = self.get_user_by_username(request.service, serializer.validated_data['username'])
        user.block.delete()
        return Response(
            self.get_serializer(dict(message=f"User {user.username} was successfully unblocked")).data
        )

    @action(detail=False, methods=['POST'], serializer_create_class=settings.SERIALIZERS.SET_PASSWORD_SERIALIZER,
            serializer_class=settings.SERIALIZERS.RETURN_SUCCESS_SERIALIZER, url_path='set-password')
    def set_password(self, request, *args, **kwargs):
        serializer = self.get_serializer_create(data=request.data)
        serializer.is_valid(raise_exception=True)
        user = self.get_user_by_username(request.service, serializer.validated_data['username'])
        user.set_password(serializer.validated_data['password'])
        user.save()
        return Response(
            self.get_serializer(dict(message=f"Password for user {user.username} was successfully set")).data
        )

    @action(detail=False, methods=['DELETE', 'POST'], url_path='(?P<username>\s+)')
    def user_update_delete(self, request, *args, **kwargs):
        user = self.get_user_by_username(request.service, kwargs['username'])
        if self.action in ['POST']:
            partial = False
            serializer = self.get_serializer_create(user, data=request.data, partial=partial)
            serializer.is_valid(raise_exception=True)
            model_obj = serializer.save()
            serializer_display = self.get_serializer(model_obj)
            return Response(serializer_display.data)
        else:
            user.delete()

        return Response(self.get_serializer(dict(message=f"User {user.username} was successfully deleted")).data)

    def get_queryset(self):
        if getattr(self.request, 'service', None):
            return super().get_queryset().filter(service=self.request.service)
        return super().get_queryset()

    @staticmethod
    def get_user_by_username(service, username):
        user = service.service_users.filter(username=username).first()

        if user is None:
            raise ValidationError(dict(username='User does not exist'))

        return user



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='drf-auth-service',
    version='1.15',
    packages=find_packages(),
    include_package_data=True,
    description='An extension of django rest framework, providing an authorization service',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://ebs-integrator.com/',
    author='EBS Integrator',
    author_email='office@ebs-integrator.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Utilities',

    ],
)



# Generated by Django 3.1.3 on 2020-11-09 12:54

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('drf_auth_service', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='userlogs',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='userblock',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='block',
                                       to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='social',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_social_identifier',
                                    to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='scope',
            name='service',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_auth_service.service'),
        ),
        migrations.AddField(
            model_name='role',
            name='scopes',
            field=models.ManyToManyField(to='drf_auth_service.Scope'),
        ),
        migrations.AddField(
            model_name='role',
            name='service',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_auth_service.service'),
        ),
        migrations.AddField(
            model_name='config',
            name='service',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='configs',
                                    to='drf_auth_service.service'),
        ),
        migrations.AddField(
            model_name='activationcode',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='activation_code',
                                    to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='ssouser',
            name='service',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='drf_auth_service.service'),
        ),
        migrations.AlterUniqueTogether(
            name='social',
            unique_together={('user', 'social_type')},
        ),
        migrations.AlterUniqueTogether(
            name='config',
            unique_together={('service', 'key')},
        ),
        migrations.AlterUniqueTogether(
            name='ssouser',
            unique_together={('username', 'service')},
        ),
    ]
